/* Tataroiu Bogdan-Cristian */

#include <stdio.h>
#include <string.h>
#include <assert.h>

#define MAXN 1000000000
#define MAXK 20
#define MAXT 20
#define MOD 666013

#define cell(j, k) ((j) * 3 + (k))
#define var_count (cell(MAXK, 2) + 1)

void mul(int A[var_count][var_count], int B[var_count][var_count]) {
    int C[var_count][var_count], i, j, k;
    memset(C, 0, sizeof(C));
    for (k = 0; k < var_count; k++) {
        for (i = 0; i < var_count; i++) {
            for (j = 0; j < var_count; j++) {
                C[i][j] = (C[i][j] + A[i][k] * (long long)B[k][j]) % MOD;
            }
        }
    }
    memcpy(A, C, sizeof(C));
}

void toPower(int res[var_count][var_count], int M[var_count][var_count],
             int power) {
    if (power == 0) {
        memset(res, 0, sizeof(int) * var_count * var_count);
        int i;
        for (i = 0; i < var_count; i++) {
            res[i][i] = 1;
        }
        return;
    }

    if (power & 1) {
        toPower(res, M, power - 1);
        mul(res, M);
    } else {
        toPower(res, M, power >> 1);
        mul(res, res);
    }
}

int main() {
    assert(freopen("ikebana.in", "rt", stdin));
    assert(freopen("ikebana.out", "wt", stdout));

    int N, K, T;
    assert(scanf("%d %d %d", &N, &K, &T) == 3);
    assert(1 <= N && N <= MAXN);
    assert(3 <= K && K <= MAXK);
    assert(4 <= T && T <= MAXT);
    /*
     * cnt[i][j][k] = numarul de aranjamente de lungime i, cu ultima gherbere
     * plasata acum j pozitii si ultima planta plasata de tipul k
     *     k == 0 pentru azalee sau orhidee
     *     k == 1 pentru hortensie
     *     k == 2 pentru celelalte tipuri de plante
     * Modelam relatia de recurenta sub forma unui sistem matriceal:
     * REC * cnt(i) = cnt(i + 1)
     * unde cnt(i) va contine elementele liniarizate sub forma unei coloane de
     * "var_count" elemente.
     */
    int init[var_count], rec[var_count][var_count], final[var_count];
    memset(init, 0, sizeof(init));
    memset(rec, 0, sizeof(rec));
    memset(final, 0, sizeof(final));

    /* Construieste starea initiala */
    init[cell(K, 0)] = 2;
    init[cell(K, 1)] = 0;
    init[cell(0, 2)] = 1;
    init[cell(K, 2)] = (T - 4);

    /* Construieste matricea de recurenta */
    int j, k;
    for (j = 1; j <= K; j++) {
        /*
          cnt(i, j, 0) += (cnt(i - 1, j - 1, 0) + cnt(i - 1, j - 1, 2)) * 2 + \
                          cnt(i - 1, j - 1, 1);
        */
        rec[cell(j, 0)][cell(j - 1, 0)] = 2;
        rec[cell(j, 0)][cell(j - 1, 2)] = 2;
        rec[cell(j, 0)][cell(j - 1, 1)] = 1;

        /*
          cnt(i, j, 1) += cnt(i - 1, j - 1, 0);
        */
        rec[cell(j, 1)][cell(j - 1, 0)] = 1;

        /*
          cnt(i, j, 2) += (cnt(i - 1, j - 1, 0) + cnt(i - 1, j - 1, 2)) * \
                          (T - 4);
        */
        rec[cell(j, 2)][cell(j - 1, 0)] = (T - 4);
        rec[cell(j, 2)][cell(j - 1, 2)] = (T - 4);
    }

    /* cnt(i, K, ?) += cnt(i - 1, K, ?) */
    /*
      cnt(i, K, 0) += (cnt(i - 1, K, 0) + cnt(i - 1, K, 2)) * 2 + \
                      cnt(i - 1, K, 1);
    */
    rec[cell(K, 0)][cell(K, 0)] = 2;
    rec[cell(K, 0)][cell(K, 2)] = 2;
    rec[cell(K, 0)][cell(K, 1)] = 1;
    /*
      cnt(i, K, 1) += cnt(i - 1, K, 0);
    */
    rec[cell(K, 1)][cell(K, 0)] = 1;
    /*
      cnt(i, 0, 2) += (cnt(i - 1, K, 0) + cnt(i - 1, K, 2));
    */
    rec[cell(0, 2)][cell(K, 0)] = 1;
    rec[cell(0, 2)][cell(K, 2)] = 1;
    /*
      cnt(i, K, 2) += (cnt(i - 1, K, 0) + cnt(i - 1, K, 2)) * (T - 4);
    */
    rec[cell(K, 2)][cell(K, 0)] = (T - 4);
    rec[cell(K, 2)][cell(K, 2)] = (T - 4);

    /* Calculeaza starea finala */
    /* final = rec ^ (N - 1) * init */
    int power[var_count][var_count];
    toPower(power, rec, N - 1);
    for (j = 0; j < var_count; j++) {
        final[j] = 0;
        for (k = 0; k < var_count; k++) {
            final[j] = (final[j] + power[j][k] * init[k]) % MOD;
        }
    }

    int SUM = 0;
    /* Sum of cnt(N, j, k), 0 <= j <= J, k in {0, 2} */
    for (j = 0; j <= K; j++) {
        for (k = 0; k < 3; k += 2) {
            if (k == 1) {
                continue;
            }
            SUM += final[cell(j, k)];
            if (SUM >= MOD) {
                SUM -= MOD;
            }
        }
    }
    printf("%d\n", SUM);

    return 0;
}
